package com.qmth.distributed.print.api;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.excel.ExcelField;
import com.qmth.distributed.print.business.bean.result.EditResult;
import com.qmth.distributed.print.business.entity.TCScoreNormal;
import com.qmth.distributed.print.business.service.TCScoreNormalService;
import com.qmth.teachcloud.common.annotation.OperationLogDetail;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.FieldUniqueEnum;
import com.qmth.teachcloud.common.enums.log.CustomizedOperationTypeEnum;
import com.qmth.teachcloud.common.util.JacksonUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.mark.entity.MarkPaper;
import com.qmth.teachcloud.mark.service.MarkPaperService;
import io.swagger.annotations.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.io.IOException;
import java.util.*;

/**
 * <p>
 * 平常作业成绩表 前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2024-02-18
 */
@Api(tags = "课程目标达成度-成绩管理-平常作业成绩Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_COURSE_DEGREE)
public class TCScoreNormalController {
    private final static Logger log = LoggerFactory.getLogger(TCScoreNormalController.class);

    @Resource
    TCScoreNormalService tcScoreNormalService;

    @Resource
    MarkPaperService markPaperService;

    @ApiOperation(value = "成绩管理列表")
    @RequestMapping(value = "/score/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "分页查询", response = Object.class)})
    public Object scoreList(@ApiParam(value = "学期ID", required = true) @RequestParam Long semesterId,
                            @ApiParam(value = "考试ID", required = true) @RequestParam Long examId,
                            @ApiParam(value = "课程编码") @RequestParam(required = false) String courseCode,
                            @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                            @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(tcScoreNormalService.scoreList(new Page<>(pageNumber, pageSize), semesterId, examId, courseCode));
    }

    @ApiOperation(value = "导入平时成绩-模板下载")
    @RequestMapping(value = "/score/template_download", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EXPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "下载成功", response = Object.class)})
    public void templateDownload(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                                 @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                                 @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                                 @ApiParam(value = "试卷类型", required = true) @RequestParam String paperType) throws IOException {
        List<ExcelField> excelFieldList = new ArrayList<>();
        excelFieldList.add(new ExcelField("examNumber", "学号", true));
        excelFieldList.add(new ExcelField("name", "姓名", true));
        excelFieldList.add(new ExcelField("score1", "作业1", true));
        excelFieldList.add(new ExcelField("score2", "作业2", true));
        excelFieldList.add(new ExcelField("score3", "作业3", false));
        tcScoreNormalService.downLoadExaminationTemplate("1、所有字段均为必填字段；\n" +
                "2、平时成绩各项数据均需要填写；\n" +
                "3、请不要删除此行，也不要删除模板中的任何列。\n" +
                "4、使用前请先删除样例数据。", "sheet1", "平时成绩导入模版", excelFieldList);
    }

    @ApiOperation(value = "导入平时成绩")
    @RequestMapping(value = "/score/import", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.IMPORT)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    @Transactional
    public Object scoreImport(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                              @ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                              @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode,
                              @ApiParam(value = "试卷编号", required = true) @RequestParam String paperNumber,
                              @ApiParam(value = "试卷类型", required = true) @RequestParam String paperType) throws IOException {
        log.debug("导入Excel开始...");
        long start = System.currentTimeMillis();
        String reqFileMd5 = ServletUtil.getRequestMd5();
        String fileMd5 = DigestUtils.md5Hex(file.getBytes());
        if (!Objects.equals(fileMd5, reqFileMd5)) {
            throw ExceptionResultEnum.MD5_EQUALS_FALSE.exception();
        }
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumberAndPaperType(examId, paperNumber, paperType);
        Objects.requireNonNull(markPaper, "未找到科目信息");

        Map<String, String> messageMap = new LinkedHashMap<>();
        try {
            StringJoiner errorData = new StringJoiner("");
            StringJoiner successData = new StringJoiner("");
            List<Map<String, String>> list = EasyExcel.read(file.getInputStream()).headRowNumber(1).sheet(0).doReadSync();

            SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
            List<TCScoreNormal> tcScoreNormalList = new ArrayList<>(list.size());
            log.info("list:{}", JacksonUtil.parseJson(list));
            Map<String, String> headMap = list.get(0);
            for (Map.Entry<String, String> entry : headMap.entrySet()) {
                if (Objects.equals(entry.getKey(), 0) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "学号")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第一行为学号");
                } else if (Objects.equals(entry.getKey(), 1) && Objects.nonNull(entry.getValue()) &&
                        !Objects.equals(entry.getValue().trim(), "姓名")) {
                    throw ExceptionResultEnum.ERROR.exception("excel表头第二行为姓名");
                }
            }

            for (int i = 1; i < list.size(); i++) {
                Map<String, String> objectMap = list.get(i);
                boolean error = false;
                JSONArray jsonArray = new JSONArray();
                TCScoreNormal tcScoreNormal = new TCScoreNormal(examId, courseCode, markPaper.getCourseName(), paperNumber, paperType, sysUser.getId());
                for (Map.Entry<String, String> entry : objectMap.entrySet()) {
                    JSONObject jsonObject = new JSONObject();
                    if (Objects.isNull(entry.getValue()) || Objects.equals(entry.getValue().trim(), "")) {
                        errorData.add("excel第" + i + "行[").add(headMap.get(entry.getKey()) + "]为空;").add("\r\n");
                        error = true;
                    } else {
                        String head = headMap.get(entry.getKey());
                        Objects.requireNonNull(head, "表头信息异常");
                        if (Objects.equals(head.trim(), "学号")) {
                            tcScoreNormal.setExamNumber(entry.getValue());
                        } else if (Objects.equals(head.trim(), "姓名")) {
                            tcScoreNormal.setName(entry.getValue());
                        } else {
                            jsonObject.put("name", headMap.get(entry.getKey()));
                            jsonObject.put("score", entry.getValue());
                            jsonArray.add(jsonObject);
                        }
                    }
                }
                if (jsonArray.size() > 0) {
                    tcScoreNormal.setScoreNormal(jsonArray.toJSONString());
                }
                if (!error) {
                    tcScoreNormalList.add(tcScoreNormal);
                }
            }
            if (!CollectionUtils.isEmpty(tcScoreNormalList)) {
                successData.add("共导入" + tcScoreNormalList.size() + "条数据");

                QueryWrapper<TCScoreNormal> tcScoreNormalQueryWrapper = new QueryWrapper<>();
                tcScoreNormalQueryWrapper.lambda().eq(TCScoreNormal::getExamId, examId)
                        .eq(TCScoreNormal::getCourseCode, courseCode)
                        .eq(TCScoreNormal::getCourseName, markPaper.getCourseName())
                        .eq(TCScoreNormal::getPaperNumber, paperNumber)
                        .eq(TCScoreNormal::getPaperType, paperType);
                tcScoreNormalService.remove(tcScoreNormalQueryWrapper);
                tcScoreNormalService.saveBatch(tcScoreNormalList);
            }
            messageMap.put("正确信息", successData.length() > 0 ? successData.toString() : "无");
            messageMap.put("错误信息", errorData.length() > 0 ? errorData.toString() : "无");
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw ExceptionResultEnum.SQL_ERROR.exception("[" + FieldUniqueEnum.convertToTitle(columnStr) + "]数据不允许重复插入");
            } else if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, ((ApiException) e).getCode(), e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        log.debug("导入Excel结束...");
        long end = System.currentTimeMillis();
        log.info("============耗时{}秒============:", (end - start) / 1000);
        return ResultUtil.ok(messageMap);
    }

    @ApiOperation(value = "平时成绩列表")
    @RequestMapping(value = "/score/normal/list", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.SEARCH)
    @ApiResponses({@ApiResponse(code = 200, message = "导入成功", response = EditResult.class)})
    public Object scoreNormalList(@ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                  @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩编辑")
    @RequestMapping(value = "/score/normal/edit", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.EDIT)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object scoreNormalEdit(@ApiParam(value = "主键", required = true) @RequestParam Long id) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩保存")
    @RequestMapping(value = "/score/normal/save", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "保存", response = EditResult.class)})
    public Object scoreNormalSave(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                  @ApiParam(value = "平时成绩json", required = true) @RequestParam String socreNormal) throws IOException {
        return ResultUtil.ok(true);
    }

    @ApiOperation(value = "平时成绩启用/禁用")
    @RequestMapping(value = "/score/normal/enable", method = RequestMethod.POST)
    @OperationLogDetail(customizedOperationType = CustomizedOperationTypeEnum.UPDATE)
    @ApiResponses({@ApiResponse(code = 200, message = "编辑", response = EditResult.class)})
    public Object scoreNormalEnable(@ApiParam(value = "主键", required = true) @RequestParam Long id,
                                    @ApiParam(value = "启用/禁用", required = true) @RequestParam Boolean enable) throws IOException {
        return ResultUtil.ok(true);
    }
}
